<ul class="nav nav-tabs card-header-tabs"><li class="nav-item"><a class="nav-link{% if active_page == 'index' %} active{% endif %}" href="./">Files</a></li><li><a class="nav-link{% if active_page == 'queue' %} active{% endif %}" href="./queue">Queue <span id="queueSize" class="badge text-bg-secondary"></span></a></li></ul>